//-----------------------------------------------------------------
// CSE 461 Project 1:                   Provided client code
//
// This file sets up the network. It creates a reliable receiver
// and sender; connects them through a SimpleUnreliableNetwork; 
// attaches send and receive applications to the sender and 
// receiver respectively. 
//
// The stop-and-wait implemntation of the reliable sender and receiver
// is used here. You might want to replace that by an implementation 
// which uses the sliding window protocol
//
// USAGE:  java project1 [seed=<value>] 
//              [bandwidth=<value>]               Mbps
//              [bandwidth-walk=<value>]
//              [no-bandwidth-variation]
//              [latency=<value>]                 ms
//              [latency-walk=<value>]
//              [no-latency-variation] 
//              [drop-prob=<value>]
//              [timeout=<value>]                 us
//              [mtu=<value>]
//              [debug="<value>"]
//-----------------------------------------------------------------
package project1;

import debug.*;
import timer.*;
import network.*;
import reliability.*; 
import application.*; 

import java.io.*;
import java.util.*;


public class project1 
{
  /*********************************************/
  // main entry point.  
  // Create an actual instance; parse the arguments and accordingly set up 
  // the network. 
  public static void main (String[] args)
  {
    project1 runProject = new project1 ();
    ParseArgs (args);
    runProject.Go (args);
  }
  
  /*********************************************/
  // Default values of parameters
  // Can be over-ridden on the command line
  public static long SEED = System.currentTimeMillis ();
  
  public static int LINK_MTU = 20;
  
  public static double LINK_BW = (0.5+0.5);
  public static double LINK_BW_SD = 0.2;
  public static double LINK_BW_WALK = 0.5;
  
  public static double LINK_LATENCY = 2.0;
  public static double LINK_LATENCY_SD = 0.5;
  public static double LINK_LATENCY_WALK = 0.5;
  
  public static double LINK_DROP_PROB = 0.25;
  
  public static long TIMEOUT = 1000;

  /*********************************************/
  // Parse the arguments and accordingly change the parameters
  private static void ParseArgs (String[] args)
  {
      for (int i=0; i<args.length; ++i)
	{
	  int eqidx = args[i].indexOf ('=');
	  if (eqidx == -1)
            continue;
	  
	  String prefix = args[i].substring (0, eqidx);
	  String value = args[i].substring (eqidx + 1);
	  
	  if (prefix.equals ("seed"))
	    SEED = ParseLong (value, 0);
	  
	  else if (prefix.equals ("bandwidth"))
            {
	      LINK_BW = ParseDouble (value, (0.5+0.5));
	      LINK_BW_SD = LINK_BW / 5.0;
            }
	  else if (prefix.equals ("bandwidth-walk"))
            LINK_BW_WALK = ParseDouble (value, (0.5+0.5));
	  
	  else if (prefix.equals ("latency"))
            {
	      LINK_LATENCY = ParseDouble (value, (0.5+0.5));
	      LINK_LATENCY_SD = LINK_LATENCY / 4.0;
            }
	  else if (prefix.equals ("latency-walk"))
            LINK_LATENCY_WALK = ParseDouble (value, (0.5+0.5));
	  
	  else if (prefix.equals ("drop-prob"))
            LINK_DROP_PROB = ParseDouble (value, 0.25);

	  else if (prefix.equals ("timeout"))
            TIMEOUT = ParseLong (value, 2000);
	  
	  else if (prefix.equals ("mtu"))
            LINK_MTU = ParseInt (value, 20);
	  
	  else if (prefix.equals ("debug"))
            Debug.SetEnabled (value);
	}
      
      for (int i=0; i<args.length; ++i)
	{
	  if (args[i].equals ("-no-bandwidth-variation"))
            LINK_BW_SD = 0.0;
	  if (args[i].equals ("-no-latency-variation"))
            LINK_LATENCY_SD = 0.0;
	}

      System.out.println("LINK_BW " + LINK_BW +
			 "\nLINK_BW_SD " + LINK_BW_SD + 
			 "\nLINK_BW_WALK " + LINK_BW_WALK + 
			 "\nLINK_LATENCY " + LINK_LATENCY + 
			 "\nLINK_LATENCY_SD " + LINK_LATENCY_SD + 
			 "\nLINK_LATENCY_WALK " + LINK_LATENCY_WALK +
			 "\nLINK_DROP_PROB " + LINK_DROP_PROB + 
			 "\nMTU " + LINK_MTU + 
			 "\nTIMEOUT " + TIMEOUT);
  }
  
  private static long ParseLong (String s, long def)
  {
    try
      {
	def = Long.parseLong (s);
      }
    catch (NumberFormatException e)
      {
      }
    
    return def;
  }
  
  private static int ParseInt (String s, int def)
  {
    try
      {
	def = Integer.parseInt (s);
      }
    catch (NumberFormatException e)
      {
      }
    
    return def;
  }
  
  private static double ParseDouble (String s, double def)
  {
    try 
      {
	def = Double.valueOf (s).doubleValue ();
      }
    catch (NumberFormatException e)
      {
      }
    catch (NullPointerException e)
      {
      }

    return def;
  }
  
  /*********************************************/
  // Set up the network 
  private void Go (String[] args)
  {
    Thread.currentThread ().setName ("Project 1 data sender");
    Timer.global.RegisterThread (Thread.currentThread ());
    synchronized (Timer.global)
      {
	Network link = new SimpleUnreliableNetwork (LINK_MTU,
			    LINK_BW, LINK_BW_SD, LINK_BW_WALK,
			    LINK_LATENCY, LINK_LATENCY_SD, LINK_LATENCY_WALK,
			    LINK_DROP_PROB, SEED);
	
	Address senderAddr   = new Address ("sender");
	Address receiverAddr = new Address ("receiver");
	
	ReliableSender   sender = new SAWSender (link, senderAddr, TIMEOUT);
	ReliableReceiver receiver = new SAWReceiver (link, receiverAddr);
	
	SimpleBytestreamSender sendApp= new SimpleBytestreamSender(sender,
							    senderAddr, 
							    receiverAddr,
								   1000); 
	SimpleBytestreamReceiver rcvApp=new SimpleBytestreamReceiver(receiver, 
								receiverAddr,
								     1000);
	
	Timer.global.UnregisterThread(Thread.currentThread());
      }
  }
}

